"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateNamespace = void 0;
const dot_prop_1 = __importDefault(require("dot-prop"));
const Exception_1 = require("../../Exception");
const Guard = __importStar(require("../Guard"));
const InferredType = __importStar(require("../InferredType"));
const Name = __importStar(require("../Name"));
const ToTypeNode = __importStar(require("../toTypeNode"));
const Reference = __importStar(require("./Reference"));
const Schema = __importStar(require("./Schema"));
const createNullableTypeNode = (factory, schema) => {
    if (!schema.type && typeof schema.nullable === "boolean") {
        const typeNode = factory.TypeNode.create({
            type: "any",
        });
        return factory.UnionTypeNode.create({
            typeNodes: [
                typeNode,
                factory.TypeNode.create({
                    type: "null",
                }),
            ],
        });
    }
};
const generateNamespace = (entryPoint, currentPoint, store, factory, schemas, context, convertContext) => {
    const basePath = "components/schemas";
    store.addComponent("schemas", {
        kind: "namespace",
        name: Name.Components.Schemas,
    });
    Object.entries(schemas).forEach(([name, targetSchema]) => {
        if (Guard.isReference(targetSchema)) {
            const schema = targetSchema;
            const reference = Reference.generate(entryPoint, currentPoint, schema);
            if (reference.type === "local") {
                const { maybeResolvedName, depth, pathArray } = context.resolveReferencePath(currentPoint, reference.path);
                // console.log({
                //   depth,
                //   pathArray,
                // });
                const createTypeNode = () => {
                    if (depth === 2) {
                        return factory.TypeReferenceNode.create({
                            name: convertContext.escapeReferenceDeclarationText(maybeResolvedName),
                        });
                    }
                    const schema = dot_prop_1.default.get(context.rootSchema, pathArray.join("."));
                    return ToTypeNode.convert(entryPoint, currentPoint, factory, schema, context, convertContext, { parent: schema });
                };
                store.addStatement(`${basePath}/${name}`, {
                    kind: "typeAlias",
                    name: convertContext.escapeDeclarationText(name),
                    value: factory.TypeAliasDeclaration.create({
                        export: true,
                        name: convertContext.escapeDeclarationText(name),
                        type: createTypeNode(),
                    }),
                });
                return;
            }
            Schema.addSchema(entryPoint, reference.referencePoint, store, factory, reference.path, reference.name, reference.data, context, convertContext);
            if (store.hasStatement(`${basePath}/${name}`, ["interface", "typeAlias"])) {
                return;
            }
            return store.addStatement(`${basePath}/${name}`, {
                kind: "typeAlias",
                name: convertContext.escapeDeclarationText(name),
                value: factory.TypeAliasDeclaration.create({
                    export: true,
                    name: convertContext.escapeDeclarationText(name),
                    comment: reference.data.description,
                    type: factory.TypeReferenceNode.create({
                        name: convertContext.escapeReferenceDeclarationText(context.resolveReferencePath(currentPoint, reference.path).name),
                    }),
                }),
            });
        }
        const schema = InferredType.getInferredType(targetSchema);
        if (!schema) {
            const typeNode = createNullableTypeNode(factory, targetSchema);
            if (!typeNode) {
                throw new Exception_1.UnSupportError("schema.type not specified \n" + JSON.stringify(targetSchema));
            }
            return typeNode;
        }
        const path = `${basePath}/${name}`;
        if (Guard.isAllOfSchema(schema)) {
            return store.addStatement(path, {
                kind: "typeAlias",
                name: convertContext.escapeDeclarationText(name),
                value: Schema.generateMultiTypeAlias(entryPoint, currentPoint, factory, name, schema.allOf, context, "allOf", convertContext),
            });
        }
        if (Guard.isOneOfSchema(schema)) {
            return store.addStatement(path, {
                kind: "typeAlias",
                name: convertContext.escapeDeclarationText(name),
                value: Schema.generateMultiTypeAlias(entryPoint, currentPoint, factory, name, schema.oneOf, context, "oneOf", convertContext),
            });
        }
        if (Guard.isAnyOfSchema(schema)) {
            return store.addStatement(path, {
                kind: "typeAlias",
                name: convertContext.escapeDeclarationText(name),
                value: Schema.generateMultiTypeAlias(entryPoint, currentPoint, factory, name, schema.anyOf, context, "anyOf", convertContext),
            });
        }
        if (Guard.isArraySchema(schema)) {
            return store.addStatement(path, {
                kind: "typeAlias",
                name: convertContext.escapeDeclarationText(name),
                value: Schema.generateArrayTypeAlias(entryPoint, currentPoint, factory, name, schema, context, convertContext),
            });
        }
        if (Guard.isObjectSchema(schema)) {
            return store.addStatement(path, {
                kind: "interface",
                name: convertContext.escapeDeclarationText(name),
                value: Schema.generateInterface(entryPoint, currentPoint, factory, name, schema, context, convertContext),
            });
        }
        if (Guard.isObjectSchema(schema)) {
            return store.addStatement(path, {
                kind: "interface",
                name: convertContext.escapeDeclarationText(name),
                value: Schema.generateInterface(entryPoint, currentPoint, factory, name, schema, context, convertContext),
            });
        }
        if (Guard.isPrimitiveSchema(schema)) {
            return store.addStatement(path, {
                kind: "typeAlias",
                name,
                value: Schema.generateTypeAlias(entryPoint, currentPoint, factory, name, schema, convertContext),
            });
        }
        throw new Exception_1.UnSupportError("schema.type = Array[] not supported. " + JSON.stringify(schema));
    });
};
exports.generateNamespace = generateNamespace;
//# sourceMappingURL=Schemas.js.map